/* memory allocation routines with error checking.
   Copyright 1989, 90, 91, 92, 93, 94 Free Software Foundation, Inc.
   modified by Takashi Uneyama <uneyama@stat.cse.nagoya-u.ac.jp> 2005

   This file is part of the libiberty library.
   Libiberty is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   Libiberty is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with libiberty; see the file COPYING.LIB.  If
   not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include "xmalloc.h"

static void xmalloc_failed(size_t size);

/* The program name if set.  */
static const char *name = "";

void xmalloc_set_program_name(const char *s)
{
    name = s;
}

void xmalloc_failed(size_t size)
{
    fprintf(stderr,
            "\n%s%sout of memory allocating %lu bytes\n",
            name, *name ? ": " : "", (unsigned long)size);
    exit(EXIT_FAILURE);
}

void *xmalloc(size_t size)
{
    void *newmem;

    if(size == 0)
        size = 1;
    newmem = malloc(size);
    if(!newmem)
        xmalloc_failed(size);

    return newmem;
}

void *xcalloc(size_t nelem, size_t elsize)
{
    void *newmem;

    if(nelem == 0 || elsize == 0)
        nelem = elsize = 1;

    newmem = calloc(nelem, elsize);
    if(!newmem)
        xmalloc_failed(nelem * elsize);

    return newmem;
}

void *xrealloc(void *oldmem, size_t size)
{
    void *newmem;

    if(size == 0)
        size = 1;
    if(!oldmem)
        newmem = malloc(size);
    else
        newmem = realloc(oldmem, size);
    if(!newmem)
        xmalloc_failed(size);

    return newmem;
}
