/*---------------------------------------------------------------------------*/

/*!  \file  xudconv.c
     \brief  the XUDF (eXtened User Definable Format) to script converter.  */

/*---------------------------------------------------------------------------*/

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#include <stdlib.h>

#include "getopt.h"
#include "tokenizer.h"
#include "analyzer.h"

/*---------------------------------------------------------------------------*/
#if defined(LUA)
#define SCRIPTNAME "lua"
#elif defined(PYTHON)
#define SCRIPTNAME "py"
#elif defined(RUBY)
#define SCRIPTNAME "rb"
#elif defined(PERL)
#define SCRIPTNAME "pl"
#else
#error LUA, PYTHON, RUBY or PERL must be defined.
#endif

/*---------------------------------------------------------------------------*/

static char programstring[] = "xudf2" SCRIPTNAME;
static char shortopts[] = "vhI:";
static struct option longopts[] =
{
    { "version", no_argument, NULL, 'v' },
    { "help", no_argument, NULL, 'h' },
    { NULL, 0, NULL, 0 }
};
static char *helpstring =
"\
Usage: %s [options]\n\
Options:\n\
  -v, --version            output version information and exit\n\
  -h, --help               display this help and exit\n\
";

static int xudf_process_options(int argc,char **argv,int *optc);
static int xudf_open_streams(int argc,char **argv,int optc,FILE **fp,FILE **fp2);

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
    int optc;
    FILE *fp,*fp2;

    if(!xudf_process_options(argc,argv,&optc))
    {
        return 1;
    }
    if(!xudf_open_streams(argc,argv,optc,&fp,&fp2))
    {
        return 1;
    }

    xudf_analyze(fp,fp2);

    return 0;
}

/*---------------------------------------------------------------------------*/
int xudf_process_options(int argc,char **argv,int *optc)
{
    int c;
    int i;

    do
    {
        i = 0;
        c = getopt_long(argc,argv,shortopts,longopts,&i);

        switch(c)
        {
            case 'v':
                    /*  version  */
                fprintf(stderr,"%s (%s) %s\n",programstring,PACKAGE,VERSION);
                exit(EXIT_SUCCESS);

            case 'h':
                    /*  help  */
                fprintf(stderr,helpstring,programstring);
                exit(EXIT_SUCCESS);

            case '?':
            case -1:
                break;

            default:
                fprintf(stderr,
                        "error: getopt returned character code 0%o\n", c);
                return XUDF_ERROR;
        }
    }
    while(c != -1);

    *optc = optind;

    return TRUE;
}

/*---------------------------------------------------------------------------*/
int xudf_open_streams(int argc,char **argv,int optc,FILE **fp,FILE **fp2)
{
    if((argc - optc) == 2)
    {
        if((*fp = fopen(argv[optc],"rt")) == NULL)
        {
            fprintf(stderr,"cannot open input file: %s\n",argv[optc]);
            return XUDF_ERROR;
        }
        if((*fp2 = fopen(argv[optc + 1],"wt")) == NULL)
        {
            fprintf(stderr,"cannot open output file: %s\n",argv[optc + 1]);
            return XUDF_ERROR;
        }
    }
    else if((argc - optc) == 1)
    {
        if((*fp = fopen(argv[optc],"rt")) == NULL)
        {
            fprintf(stderr,"cannot open input file: %s\n",argv[optc]);
            return XUDF_ERROR;
        }
        *fp2 = stdout;
    }
    else if((argc - optc) == 0)
    {
        *fp = stdin;
        *fp2 = stdout;
    }
    else
    {
        fprintf(stderr,helpstring,programstring);
        return XUDF_ERROR;
    }

    return TRUE;
}

/*---------------------------------------------------------------------------*/
