/*---------------------------------------------------------------------------*/

/*!  \file  alist.c
     \brief  associated list for macro definition.  */

/*---------------------------------------------------------------------------*/

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xmalloc.h"

#include "alist.h"

/*---------------------------------------------------------------------------*/
xudf_definition_alist *xudf_new_definition_alist(char *name,char *definition)
{
    xudf_definition_alist *xalist;

    xalist = xcalloc(1,sizeof(xudf_definition_alist));

    xalist->name = xstrdup(name);
    xalist->definition = xstrdup(definition);

    return xalist;
}

/*---------------------------------------------------------------------------*/
xudf_definition_alist *xudf_add_definition_alist(xudf_definition_alist *xalist,
                                                 char *name,char *definition)
{
    xudf_definition_alist *xalistlast;

    xalistlast = xalist;
    while(xalistlast->next != NULL)
    {
        xalistlast = xalistlast->next;
    }

    xalistlast->next = xcalloc(1,sizeof(xudf_definition_alist));
    xalistlast->next->name = xstrdup(name);
    xalistlast->next->definition = xstrdup(definition);

    return xalistlast;
}

/*---------------------------------------------------------------------------*/
xudf_definition_alist *xudf_insert_definition_alist(xudf_definition_alist *xalist,
                                                    char *name,char *definition)
{
    xudf_definition_alist *xalistfirst;

    xalistfirst = xcalloc(1,sizeof(xudf_definition_alist));
    xalistfirst->name = xstrdup(name);
    xalistfirst->definition = xstrdup(definition);

    xalistfirst->next = xalist;

    return xalistfirst;
}

/*---------------------------------------------------------------------------*/
void xudf_free_definition_alist(xudf_definition_alist *xalist)
{
    if(xalist != NULL)
    {
        xudf_free_definition_alist(xalist->next);
        if(xalist->name != NULL)
            xfree(xalist->name);
        if(xalist->definition != NULL)
            xfree(xalist->definition);
        xfree(xalist);
    }
}

/*---------------------------------------------------------------------------*/
void xudf_print_definition_alist(FILE *fp,xudf_definition_alist *xalist)
{
    while(xalist != NULL)
    {
        fprintf(fp,"\\%s = %s\n",xalist->name,xalist->definition);
        xalist = xalist->next;
    }
}

/*---------------------------------------------------------------------------*/
char *xudf_find_definition_from_definition_alist(xudf_definition_alist *xalist,
                                                 char *name)
{
    while(xalist != NULL)
    {
        if(strcmp(xalist->name,name) == 0)
            return xalist->definition;

        xalist = xalist->next;
    }

    return NULL;
}

/*---------------------------------------------------------------------------*/
char *xudf_find_definition_from_definition_alist_or_exit(xudf_definition_alist *xalist,
                                                         char *name)
{
    char *definition;

    definition = xudf_find_definition_from_definition_alist(xalist,name);

    if(definition != NULL)
        return definition;

    fprintf(stderr,"error: cannot find macro named `%s'\n",name);
    exit(EXIT_FAILURE);
}

/*---------------------------------------------------------------------------*/
