/*---------------------------------------------------------------------------*/

/*!  \file  list.c
     \brief  include path list.  */

/*---------------------------------------------------------------------------*/

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xmalloc.h"

#include "list.h"

/*---------------------------------------------------------------------------*/
xudf_path_list *xudf_new_path_list(char *path)
{
    xudf_path_list *xlist;

    xlist = xcalloc(1,sizeof(xudf_path_list));
    xlist->path = xstrdup(path);

    return xlist;
}

/*---------------------------------------------------------------------------*/
xudf_path_list *xudf_add_path_list(xudf_path_list *xlist,char *path)
{
    xudf_path_list *xlistlast;

    xlistlast = xlist;
    while(xlistlast->next != NULL)
    {
        xlistlast = xlistlast->next;
    }

    xlistlast->next = xcalloc(1,sizeof(xudf_path_list));
    xlistlast->next->path = xstrdup(path);

    return xlistlast;
}

/*---------------------------------------------------------------------------*/
xudf_path_list *xudf_insert_path_list(xudf_path_list *xlist,char *path)
{
    xudf_path_list *xlistfirst;

    xlistfirst = xcalloc(1,sizeof(xudf_path_list));
    xlistfirst->path = xstrdup(path);

    xlistfirst->next = xlist;

    return xlistfirst;
}

/*---------------------------------------------------------------------------*/
void xudf_free_path_list(xudf_path_list *xlist)
{
    if(xlist != NULL)
    {
        xudf_free_path_list(xlist->next);
        if(xlist->path != NULL)
            xfree(xlist->path);
        xfree(xlist);
    }
}

/*---------------------------------------------------------------------------*/
void xudf_print_path_list(FILE *fp,xudf_path_list *xlist)
{
    while(xlist != NULL)
    {
        fprintf(fp,"%s ",xlist->path);
        xlist = xlist->next;
    }
    fprintf(fp,"\n");
}

/*---------------------------------------------------------------------------*/
char *xudf_find_file_from_path_list(xudf_path_list *xlist,char *filename)
{
    FILE *fp;
    int i;
    static char filename_buffer[XUDF_FILENAME_SIZE];

    fp = fopen(filename,"rt");
    if(fp != NULL)
    {
        fclose(fp);
        return filename;
    }

    while(xlist != NULL)
    {
        if(strlen(xlist->path) > 0)
        {
            strcpy(filename_buffer,xlist->path);

            i = strlen(filename_buffer);
            if(filename_buffer[i] != '/')
            {
                filename_buffer[i] = '/';
                filename_buffer[i + 1] = '\0';
            }

            strcat(filename_buffer,filename);

            fp = fopen(filename_buffer,"rt");
            if(fp != NULL)
            {
                fclose(fp);
                return filename_buffer;
            }
        }

        xlist = xlist->next;
    }

    return NULL;
}

/*---------------------------------------------------------------------------*/
char *xudf_find_file_from_path_list_or_exit(xudf_path_list *xlist,char *filename)
{
    char *path;

    path = xudf_find_file_from_path_list(xlist,filename);

    if(path != NULL)
        return path;

    fprintf(stderr,"error: cannot find the file `%s'\n",filename);
    exit(EXIT_FAILURE);
}

/*---------------------------------------------------------------------------*/
