/*---------------------------------------------------------------------------*/

/*!  \file  tokenizer.h
     \brief  XUDF (eXtened User Definable Format) tokenizer.  */

/*---------------------------------------------------------------------------*/

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_TOKENIZER_H
#define INCLUDE_TOKENIZER_H

#include <stdio.h>

#define XUDF_TOKEN_SIZE 0x1000

/*---------------------------------------------------------------------------*/
typedef enum xudf_token_type
{
    TT_UNKNOWN = -1,
    TT_BACKSLASH = 1,
    TT_LEFT_BRACE,
    TT_RIGHT_BRACE,
    TT_LEFT_BRACKET,
    TT_RIGHT_BRACKET,
    TT_COLON,
    TT_GREATER_THAN,
    TT_LESS_THAN,
    TT_BEGIN,
    TT_END,
    TT_HEADER,
    TT_DEF,
    TT_DATA,
    TT_UNIT,
    TT_GLOBAL_DEF,
    TT_RECORD,
    TT_INCLUDE,
    TT_DEFINE,
    TT_STRING,
    TT_SPECIFIER,
    TT_OTHER,
    TT_COUNT
} xudf_token_type;

typedef struct xudf_token
{
    xudf_token_type type;
    char *string;
    int newlines;
} xudf_token;

/*---------------------------------------------------------------------------*/
xudf_token *xudf_create_token(void);
void xudf_destroy_token(xudf_token *xt);

int xudf_get_token(FILE *fp,xudf_token *xt);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
