/*---------------------------------------------------------------------------*/

/*!  \file  xudfpp.c
     \brief  the XUDF (eXtened User Definable Format) preprocessor.  */

/*---------------------------------------------------------------------------*/

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "getopt.h"
#include "tokenizer.h"
#include "analyzer.h"
#include "list.h"
#include "alist.h"

/*---------------------------------------------------------------------------*/
static char programstring[] = "xudfpp";
static char shortopts[] = "vhI:D:";
static struct option longopts[] =
{
    { "version", no_argument, NULL, 'v' },
    { "help", no_argument, NULL, 'h' },
    { NULL, 0, NULL, 0 }
};
static char *helpstring =
"\
Usage: %s [options]\n\
Options:\n\
  -I <directory>           add <directory> to the search paths\n\
  -D <name>[=<definition>] define macro <name> (= <definition>)\n\
  -v, --version            output version information and exit\n\
  -h, --help               display this help and exit\n\
";

/*---------------------------------------------------------------------------*/
static int xudf_process_options(int argc,char **argv,int *optc,
                                xudf_path_list *xlist,xudf_definition_alist *xalist);
static int xudf_open_streams(int argc,char **argv,int optc,FILE **fp,FILE **fp2);

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
    int optc;
    FILE *fp,*fp2;
    xudf_path_list *xlist;
    xudf_definition_alist *xalist;

    xlist = xudf_new_path_list("");
    xalist = xudf_new_definition_alist("","");

    if(!xudf_process_options(argc,argv,&optc,xlist,xalist))
    {
        return EXIT_FAILURE;
    }
    if(!xudf_open_streams(argc,argv,optc,&fp,&fp2))
    {
        return EXIT_FAILURE;
    }

    xudf_process_macro(fp,fp2,xlist,xalist);

/*     xudf_print_path_list(stderr,xlist); */
/*     xudf_print_definition_alist(stderr,xalist); */

    xudf_free_path_list(xlist);
    xudf_free_definition_alist(xalist);

    return 0;
}

/*---------------------------------------------------------------------------*/
int xudf_process_options(int argc,char **argv,int *optc,
                         xudf_path_list *xlist,xudf_definition_alist *xalist)
{
    int c;
    int i;
    char *substr;

    do
    {
        i = 0;
        c = getopt_long(argc,argv,shortopts,longopts,&i);

        switch(c)
        {
            case 'v':
                    /*  version  */
                fprintf(stderr,"%s (%s) %s\n",programstring,PACKAGE,VERSION);
                exit(EXIT_SUCCESS);

            case 'h':
                    /*  help  */
                fprintf(stderr,helpstring,programstring);
                exit(EXIT_SUCCESS);

            case 'I':
                    /*  include path  */
                xudf_add_path_list(xlist,optarg);
                break;

            case 'D':
                if((substr = strstr(optarg,"=")) == NULL)
                {
                    xudf_add_definition_alist(xalist,optarg,"");
                }
                else
                {
                    *substr++ = '\0';
                    xudf_add_definition_alist(xalist,optarg,substr);
                    *(--substr) = '=';
                }
                break;

            case '?':
            case -1:
                break;

            default:
                fprintf(stderr,
                        "error: getopt returned character code 0%o\n", c);
                return XUDF_ERROR;
        }
    }
    while(c != -1);

    *optc = optind;

    return TRUE;
}

/*---------------------------------------------------------------------------*/
int xudf_open_streams(int argc,char **argv,int optc,FILE **fp,FILE **fp2)
{
    if((argc - optc) == 2)
    {
        if((*fp = fopen(argv[optc],"rt")) == NULL)
        {
            fprintf(stderr,"cannot open input file: %s\n",argv[optc]);
            return XUDF_ERROR;
        }
        if((*fp2 = fopen(argv[optc + 1],"wt")) == NULL)
        {
            fprintf(stderr,"cannot open output file: %s\n",argv[optc + 1]);
            return XUDF_ERROR;
        }
    }
    else if((argc - optc) == 1)
    {
        if((*fp = fopen(argv[optc],"rt")) == NULL)
        {
            fprintf(stderr,"cannot open input file: %s\n",argv[optc]);
            return XUDF_ERROR;
        }
        *fp2 = stdout;
    }
    else if((argc - optc) == 0)
    {
        *fp = stdin;
        *fp2 = stdout;
    }
    else
    {
        fprintf(stderr,helpstring,programstring);
        return XUDF_ERROR;
    }

    return TRUE;
}

/*---------------------------------------------------------------------------*/
